% Quantum mechanical scattering of a Gaussian wave packet at a barrier in two
% dimensions;
% Possible configurations of the barrier:
%  potential step; barrier; barrier with multiple slits
%  help set_pot gives more information;
% Basic calculation is done here:
%    Definition of  parameters; arrays of sampling points; 
%    calculation of the potential (this is done with the routine set_pot;
%    help set_pot gives more information) and the start function; 
%    memory prealloation for arrays containing the results; 
%    execution of the time evolution; 
%    writing a structure array containint the data necessary to
%    perform the animation; calling the animation routine;
% The natural units system is used, as listed on the following site:
%       http://physics.nist.gov/cgi-bin/cuu/Results?search_for=natural
% Visualisation:
%   To visualise complex numbers a colormap is used that associates every
%   complex number with a unique color. The color map uses the HLS color
%   system (hue-lightness-saturation). The argument(z) is given by the
%   Hue. The lightness of the color gives the squared absolute
%   value. The saturation is arbitrary. It has the following effect:
%     Saturation = 1 --> strong colours
%     Saturation = 0 --> grey scaled 
%   A better (fast) description can be found here:
%       http://www.kfunigraz.ac.at/imawww/vqm/pages/complex/index.html
%   A detailed description can be found in the following book:
%       Thaller Bernd, Visual quantum mechanics, New York, Springer 2000
%   There is also a script called complex_plane_in_hls.m that shows the
%   complex plane visulised with this colormap. 
% Two operating modes:
%   This script first prompts the user, if time and memory should be saved.
%   If 1 is chosen a faster and memory saving calculation is used. This
%   calculation only saves the differences between two timesteps, converts
%   them into an rgb matrix and animates by writing this values (starting
%   from the matrix at t=0) back into the matrix. 
%   The other way of calculation, calculates a full matrix of values at
%   every timestep. When animation is done every matrix is converted into
%   rgb values and plotted.  
%   This way of calculation is very slow and needs lots of memory. If a
%   grid with 128 sampling points is used only the optimised method is reasonable

%clear all;
%compress = input('Should we save time and memory? 1...yes 0...no '); 

function AnimationData = Quantenstreuung2D()

% get guidata
qs_main = findobj(0,'Tag','QS_MAIN');
myhandles = guidata(qs_main);

compress = 0;

h_bar = 1;                      
mass  = 1;                      

grid_range       = 25;
nr_of_gridpoints = 128;
step_width       = 2 * grid_range / nr_of_gridpoints;

kinetic_energy = 20;

x_wavenumber_t0 =  sqrt(2*mass*kinetic_energy/h_bar^2);
y_wavenumber_t0 =  0;
x_position_t0   = -5;
y_position_t0   = -step_width;  % shifting the starting position to hit the potential barrier symmetrically
t0 = 0;

%[x,y] array of sampling-points
[x,y] = meshgrid(-grid_range:step_width:grid_range-step_width);

% Evaluating the wavenumbers:
delta_wavenumber =  2*pi / (nr_of_gridpoints*step_width);
wavenumber_left  =  0:delta_wavenumber:(nr_of_gridpoints/2-1)*delta_wavenumber;
wavenumber_right = -(nr_of_gridpoints/2*delta_wavenumber):delta_wavenumber:-1*delta_wavenumber;
wavenumber       =  [wavenumber_left,wavenumber_right];
wavenumber_control = max(wavenumber);
[kx,ky] = meshgrid(wavenumber);

% checking if the wavenumber corresponding to the kinetic energy lies
% within the nyquist limit
if x_wavenumber_t0 > wavenumber_control | y_wavenumber_t0 > wavenumber_control
  error('The kinetic energy value is to high')
end

% producing the potential barrier:
NrOfSlits = myhandles.numslits;            
SlitWidth = myhandles.slitwidth;      
SlitDist  = myhandles.slitdist;
PotHeight = myhandles.potheight;     
PotWidth  = myhandles.potwidth;      
CloseOneSlit = myhandles.closeoneslit;

[potential,EffSlitWidth,EffSlitDist,EffPotWidth] = set_pot(nr_of_gridpoints,grid_range,NrOfSlits,SlitWidth,SlitDist,PotHeight,PotWidth,CloseOneSlit);


% operator of the kinetic energy (in momentum space)
kinetic_energy_operator = (h_bar^2.*(kx.^2+ky.^2))./(2*mass);


% start function (Gaussian)
width = 1.5;
wavefunction_0 = (2*pi*width^2)^(-1/4)*exp(i*(x_wavenumber_t0*x+y_wavenumber_t0*y)).*exp(-(((x-x_position_t0).^2.+(y-y_position_t0).^2)/(4*width^2)));
start_wavefunction = wavefunction_0;

% increment of time and number of time steps
time_step = 5e-3;
nr_of_timesteps = 320;

% Preallocating memory for the results; depending on the selected method
if compress == 0;
  wavefunction = zeros(nr_of_gridpoints,nr_of_gridpoints,nr_of_timesteps+1);
elseif compress == 1;
  index          = zeros(1 , 1/2 * nr_of_gridpoints^2 * nr_of_timesteps);
  index_to_index = zeros( nr_of_timesteps + 1);
  wavefunction = zeros(1, 1/2 * nr_of_gridpoints^2 * nr_of_timesteps);
  start_index = 0;
end


                     %%%%%%%%%%%%%%%%%%%%__Time evolution__%%%%%%%%%%%%%%%%%%%%%%%

% Evolution factor potential
expV = exp(-i * potential * time_step / (2*h_bar));
% Evolution factor kinetic energy
expT = exp(-i * kinetic_energy_operator * time_step / h_bar);

inc = 0;
for t = 0:time_step:nr_of_timesteps*time_step;
  inc = inc + 1;
  
  % applying the potential operator
  phi_x_0 = expV .* wavefunction_0;
  
  % transformation into k-space
  phi_k_0 = fft2(phi_x_0);
  
  % applying kinetic operator in k-space
  phi_k_1 = expT .* phi_k_0;
  
  % back transformation into configuration space
  phi_x_1 = ifft2(phi_k_1);
  
  % applying the potential operator again
  wavefunction_1 = expV .* phi_x_1;
  
  if compress == 0;
    wavefunction(:,:,inc) = wavefunction_1;
  elseif compress == 1;
    [nr_of_diff_values , pos_of_diff_values, diff_values] = compress_matrices(wavefunction_0 , wavefunction_1); 
    index_to_index(inc) = start_index + nr_of_diff_values;
    index(start_index + 1 : start_index + nr_of_diff_values) = pos_of_diff_values;
    wavefunction(start_index + 1 : start_index +  nr_of_diff_values) = diff_values;
    start_index = start_index + nr_of_diff_values;
    if start_index >  1/2 * nr_of_gridpoints^2 * nr_of_timesteps
      warning('The number of differences is too high; calculating with "optimisation" is not reasonable');
    end
  end
  
  wavefunction_0 = wavefunction_1;
end

% if index and wavefunction were prealloated too long, they are cut
if compress == 1;
  index_mat = find(index == 0);
  index = index(1:index_mat(1) - 1);
  wavefunction = wavefunction(1 : index_mat(1) - 1);
end

% deleting arrays which are no longer used
clear phi_x_0 phi_k_0 phi_k_1 phi_x_1 psi_x_1 wavefunction_0 wavefunction_1

AnimationData.potential          = potential;
AnimationData.nr_of_gridpoints   = nr_of_gridpoints;
AnimationData.wavefunction       = wavefunction;
AnimationData.start_wavefunction = start_wavefunction;
AnimationData.nr_of_timesteps    = nr_of_timesteps;
if compress == 1;
  AnimationData.index            = index;
  AnimationData.index_to_index   = index_to_index;
end

myhandles.animation_data = AnimationData;

% again deleting arrays which are no longer used
clear start_wavefunction wavefunction potential 


% save guidata
guidata(qs_main,myhandles);


end
